<?php
namespace Teknisa\COZ\Controllers;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DTO\Response\Error;

class Coz01000ConsumoPlanejado extends \Zeedhi\Framework\Controller\Report {

	protected $crudService;
	protected $util;
	protected $dataSourceManager;
	protected $novoCodigo;
	protected $entityManager;
	protected $birtStrategy;

	public function __construct(ReportService $reportService, $crudService, $util, $novoCodigo, $entityManager, $birtStrategy) {
		$this->crudService    	= $crudService;
		$this->util             = $util;
		$this->novoCodigo    	= $novoCodigo;
		$this->entityManager    = $entityManager;
		$this->birtStrategy     = $birtStrategy;
		
		parent::__construct($reportService);
    }
	
	protected $reportMapping = array (
      'cus01000' => 
        array (
            'strategy' => 'BIRT',
            'parameterMapping' => 
            array (
                'DTINICIAL'     => 'P_DTINI',
                'DTFINAL'       => 'P_DTFIN',
                'CDCENTRAL'     => 'P_CDCENTRAL',
                'NRLISTA'       => 'P_NRLISTA',
                'NRCONSPLAN'    => 'P_NRCONSPLAN',
                'ORDENA'        => 'P_ORDENA',
                'CDPICTPROD'    => 'P_CDPICTPROD',
                'CDARVPRODINI'  => 'P_CDARVPRODINI',
                'CDARVPRODFIN'  => 'P_CDARVPRODFIN',
                'NRORG'         => 'P_NRORG'
            )
        ),
        'cus01001' => 
        array (
            'strategy' => 'BIRT',
            'parameterMapping' => 
            array (
                'DTINICIAL'     => 'P_DTINI',
                'DTFINAL'       => 'P_DTFIN',
                'CDCENTRAL'     => 'P_CDCENTRAL',
                'NRLISTA'       => 'P_NRLISTA',
                'NRCONSPLAN'    => 'P_NRCONSPLAN',
                'ORDENA'        => 'P_ORDENA',
                'CDPICTPROD'    => 'P_CDPICTPROD',
                'CDARVPRODINI'  => 'P_CDARVPRODINI',
                'CDARVPRODFIN'  => 'P_CDARVPRODFIN',
                'NRORG'         => 'P_NRORG'
            )
        ),
        'cus01002' => 
        array (
            'strategy' => 'BIRT',
            'parameterMapping' => 
            array (
                'DTINICIAL'     => 'P_DTINI',
                'DTFINAL'       => 'P_DTFIN',
                'CDCENTRAL'     => 'P_CDCENTRAL',
                'NRLISTA'       => 'P_NRLISTA',
                'NRCONSPLAN'    => 'P_NRCONSPLAN',
                'ORDENA'        => 'P_ORDENA',
                'CDPICTPROD'    => 'P_CDPICTPROD',
                'CDARVPRODINI'  => 'P_CDARVPRODINI',
                'CDARVPRODFIN'  => 'P_CDARVPRODFIN',
                'NRORG'         => 'P_NRORG'
            )
        )
    );
	
	public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response){
	    try{
	        
	        $row = $request->getRow();

    	    if(empty($row['NMSERVFILI'])) $row['NMSERVFILI'] = $this->filtraServicos($row);
	    	
    	    $row['NRLISTA'] = $this->gerarListaServ($row['NMSERVFILI'], $row['NRORG']);
    
    	    $nrListaConsplan = $this->gerarLista($row['NMSERVFILI'], $row['NRORG']);

    	    $cdprodini = empty($row['CDARVPRODINI']) ? '0' : $row['CDARVPRODINI'];
            if(empty($row['CDARVPRODFIN'])) $cdprodfin = 'Z';
            else $cdprodfin = str_pad($row['CDARVPRODFIN'], 10, "Z", STR_PAD_RIGHT);
            
            $row['CDARVPRODINI'] = $cdprodini;
            $row['CDARVPRODFIN'] = $cdprodfin;
            
    	    $row['NRCONSPLAN'] = $this->gerarNrConsplan($nrListaConsplan, $row);
    	    $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
	    }catch(\Exception $e){
	        $response->setError(new Error($e->getMessage(), 500));
	    }
	}
	
	public function filtraServicos($row){
		 $conditions = [
			 	array(
                    "columnName" => "NRORG",
                    "operator"   => '=',
                    "value"      =>  $this->crudService->getEnvironment()->getNrOrg()
                ),
                array(
                    "columnName" => 'CDCENTRAL',
                    "operator"   => '=',
                    "value"      => $row['CDCENTRAL']
                ),
                array(
                    "columnName" => 'CDGRUPFILI',
                    "operator"   => '=',
                    "value"      => $row['CDGRUPFILI']  === '' ? 'T' : $row['CDGRUPFILI']
                ),
                array(
                    "columnName" => 'CDGRUPSERV',
                    "operator"   => '=',
                    "value"      => $row['NMGRUPSERV'] === '' || sizeof($row['NMGRUPSERV']) == 0 ? 'T' : join('_',$row['NMGRUPSERV'])   
                ),
                array(
                    "columnName" => 'CDFILIAGREGA',
                    "operator"   => '=',
                    "value"      => $row['NMFILIAL'] === '' || sizeof($row['NMFILIAL']) == 0 ? 'T' : join('_',$row['NMFILIAL'])
                ),
                
            ];
         $dataset = $this->crudService->getFilteredDataSet("servico_por_filiais", $conditions);       
 		 $servicos = $dataset->getRows();
 		 
 		 return $servicos;
	}
	
	public function gerarListaServ($servicos, $nrorg) {
     try {   
            
        	$nrlista = $this->novoCodigo->getCodeByTableName('LISTA_SERVICO', $nrorg);
        	$sql = "INSERT INTO LISTA_SERVICO (NRLISTA, CDFILIAL, CDSERVFILI, NMSERVICO, CDGRUPSERV)
        	                    VALUES (:NRLISTA, :CDFILIAL, :CDSERVFILI, :NMSERVICO, :CDGRUPSERV)";
            
        	foreach ($servicos as $servico) {
                $stmt = $this->entityManager->getConnection()->prepare($sql);
                $stmt->execute(
                    array(
                        "NRLISTA"    => $nrlista,
                        "CDFILIAL"   => $servico['CDFILIAL'],
                        "CDSERVFILI" => $servico['CDSERVFILI'],
                        "NMSERVICO"  => $servico['NMSERVICO'],
                        "CDGRUPSERV" => empty($servico['CDGRUPSERV']) ? null : $servico['CDGRUPSERV']
                    )
                );
            }
            return $nrlista;
        } catch (\Exception $e) {
            throw $e;
        }
    }
	
	public function gerarLista($servicos, $nrorg) {
     try {  
        	$nrlista = $this->novoCodigo->getCodeByTableName('LISTA', $nrorg);
        	$sql = "INSERT INTO LISTA (NRLISTA, CDLISTA)
        	                    VALUES (:NRLISTA, :CDLISTA)";
            
        	foreach ($servicos as $servico) {
                $stmt = $this->entityManager->getConnection()->prepare($sql);
                $stmt->execute(
                    array(
                        "NRLISTA" => $nrlista,
                        "CDLISTA" => $servico['CDFILIAL'].$servico['CDSERVFILI']
                    )
                );
            }
            return $nrlista;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
	public function gerarNrConsplan($nrlista, $row) {
		try {
		    $procedure = new StoredProcedure($this->entityManager->getConnection(), 'CONSPLAN_GERAL');
            $procedure->addParam(new Param('NRLISTA', Param::PARAM_INPUT, $nrlista, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('DTINICIAL', Param::PARAM_INPUT, $row['DTINICIAL'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('DTFINAL', Param::PARAM_INPUT, $row['DTFINAL'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('IDPRODUTOS', Param::PARAM_INPUT, 'CFP', Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('IDSEPPRATO', Param::PARAM_INPUT, 'P', Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('PEACRESCIMO', Param::PARAM_INPUT, 0, Param::PARAM_TYPE_INT));
            $procedure->addParam(new Param('IDARREDOND', Param::PARAM_INPUT, 'N', Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('CDPRODUTOINI', Param::PARAM_INPUT, '0', Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('CDPRODUTOFIN', Param::PARAM_INPUT, 'Z', Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('CDOPERADOR', Param::PARAM_INPUT, $this->crudService->getEnvironment()->getCdOperador(), Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('NRCONSPLANTH', Param::PARAM_INPUT, 'GERANOVCOD', Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('NRCONSPLAN', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 10));
            $nrconsplan = $procedure->execute()['NRCONSPLAN'];
            return $nrconsplan;
		} catch (\Exception $e) {
			throw $e;
    	}
	}
	
	public function excluirParametros(DTO\Request\Row $request, DTO\Response $response){
	    $row = $request->getRow();
	   if(isset($row["NRLISTA"])){
	        $this->util->deleteLista($row["NRLISTA"]);
    	    $connection = $this->entityManager->getConnection();
            $query="DELETE FROM CONSPLAN WHERE NRORG = :NRORG AND NRCONSPLAN = :NRCONSPLAN";
            $connection->executeQuery($query, array("NRORG" => $row['NRORG'], "NRCONSPLAN" => $row['NRCONSPLAN']));
	   }
	   
	}
}